﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework;
using System.IO;
using System.Xml;

namespace SpriteSheetPipeline
{
    /// <summary>
    /// Class responsible for importing sprite sheet XML files
    /// </summary>
    [ContentImporter(".spritesheet", DefaultProcessor = "SpriteSheetProcessor",
      DisplayName = "Spritesheet Importer")]
    class SpriteSheetImporter : ContentImporter<SpriteSheetMiddleman>
    {
        /// <summary>
        /// Reader for the file.
        /// </summary>
        XmlTextReader read;

        private string lastFile;
        private int lastLength;

        SpriteSheetMiddleman ret;

        public override SpriteSheetMiddleman Import(string filename, ContentImporterContext context)
        {
            //Load spritesheet information from the XML file
            read = new XmlTextReader(filename);
            read.WhitespaceHandling = WhitespaceHandling.None;
            read.MoveToContent();

            ret = new SpriteSheetMiddleman();

            //Structure set up
            string thisNode = "spritesheet";
            
            //Read "spritesheet" node
            while (read.Read())
            {
                if (read.NodeType == XmlNodeType.Element)
                {
                    switch (read.LocalName.ToLower())
                    {
                        case "animation":
                            ret.AddAnimation(ReadAnimation());
                            break;
                        case "file":
                            ret.AddFile(ReadFile());
                            break;
                        default:
                            thisNode = read.LocalName.ToLower();
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.Text)
                {
                    switch (thisNode)
                    {
                        
                    }
                }
            }

            read.Close();

            //Return
            return ret;
        }


        /// <summary>
        /// Read one "animation" element from the XML document.
        /// </summary>
        private Animation ReadAnimation()
        {
            Animation building = new Animation();
            string thisNode = "animation";

            while (read.Read())
            {
                if (read.NodeType == XmlNodeType.Element)
                {
                    switch (read.LocalName.ToLower())
                    {
                        case "frame":
                            building.AddFrame(ReadFrame());
                            break;
                        default:
                            thisNode = read.LocalName.ToLower();
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.Text)
                {
                    switch (thisNode)
                    {
                        case "name":
                            building.name = read.Value;
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.EndElement)
                {
                    if (read.LocalName.ToLower() == "animation")
                        return building;
                }
            }

            //Unreachable code in properly-formatted XML
            return null;
        }


        /// <summary>
        /// Read one "animation" element from the XML document.
        /// </summary>
        private File ReadFile()
        {
            File building = new File();
            string thisNode = "file";

            while (read.Read())
            {
                if (read.NodeType == XmlNodeType.Element)
                {
                    switch (read.LocalName.ToLower())
                    {
                        default:
                            thisNode = read.LocalName.ToLower();
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.Text)
                {
                    switch (thisNode)
                    {
                        case "filename":
                            building.name = read.Value;
                            break;
                        case "normals":
                            building.normals = read.Value;
                            break;
                        case "width":
                            building.width = int.Parse(read.Value);
                            break;
                        case "height":
                            building.height = int.Parse(read.Value);
                            break;
                        case "offsetx":
                            building.offx = int.Parse(read.Value);
                            break;
                        case "offsety":
                            building.offy = int.Parse(read.Value);
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.EndElement)
                {
                    if (read.LocalName.ToLower() == "file")
                        return building;
                }
            }

            //Unreachable code in properly-formatted XML
            return null;
        }


        /// <summary>
        /// Read one "frame" element from the XML document.
        /// </summary>
        private Frame ReadFrame()
        {
            Frame building = new Frame();
            string thisNode = "frame";

            while (read.Read())
            {
                if (read.NodeType == XmlNodeType.Element)
                {
                    switch (read.LocalName.ToLower())
                    {
                        default:
                            thisNode = read.LocalName.ToLower();
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.Text)
                {
                    switch (thisNode)
                    {
                        case "id":
                            building.id = Convert.ToInt32(read.Value);
                            break;
                        case "length":
                            building.length = Convert.ToInt32(read.Value);
                            lastLength = building.length;
                            break;
                        case "filename":
                            building.file = read.Value;
                            lastFile = building.file;
                            break;
                    }
                }
                else if (read.NodeType == XmlNodeType.EndElement)
                {
                    if (read.LocalName.ToLower() == "frame")
                    {
                        //Fill in defaults
                        if (building.length == 0)
                            building.length = lastLength;
                        if (building.file == null || building.file == "")
                            building.file = lastFile;

                        return building;
                    }
                }
            }

            //Unreachable code in properly-formatted XML
            return null;
        }
    }
}
