﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace SpriteSheetPipeline
{
    /// <summary>
    /// Storage class for sprite sheet files loaded through the content pipeline
    /// </summary>
    class SpriteSheetMiddleman
    {
        /// <summary>
        /// The files used by this spritesheet
        /// </summary>
        public List<File> files;

        /// <summary>
        /// The animations drawn from this spritesheet.
        /// </summary>
        public List<Animation> animations;

        /// <summary>
        /// Construct storage structures.
        /// </summary>
        public SpriteSheetMiddleman()
        {
            animations = new List<Animation>();
            files = new List<File>();
        }

        /// <summary>
        /// Adds a new animation to this spritesheet.
        /// </summary>
        /// <param name="animation"></param>
        public void AddAnimation(Animation animation)
        {
            animations.Add(animation);
        }

        /// <summary>
        /// Adds a new file to this spritesheet and returns the index
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        public void AddFile(File file)
        {
            files.Add(file);
        }
    }


    class File
    {
        public string name = "";

        public string normals = "";

        public int width;

        public int height;

        public int offx;

        public int offy;

        public File()
        {

        }
    }


    /// <summary>
    /// Represents one animation in the spritesheet.
    /// </summary>
    class Animation
    {
        /// <summary>
        /// The name of this animation.
        /// </summary>
        public string name;

        /// <summary>
        /// The frames that make up this animation.
        /// </summary>
        public List<Frame> frames;

        /// <summary>
        /// Construct storage structures.
        /// </summary>
        public Animation()
        {
            frames = new List<Frame>();
        }

        /// <summary>
        /// Add a new frame to this animation.
        /// </summary>
        /// <param name="frame"></param>
        public void AddFrame(Frame frame)
        {
            frames.Add(frame);
        }
    }


    /// <summary>
    /// Represents one frame in an animation.
    /// </summary>
    class Frame
    {
        /// <summary>
        /// The location of this frame on the image.
        /// </summary>
        public int id;

        /// <summary>
        /// The length of this frame, in milliseconds.
        /// </summary>
        public int length;

        /// <summary>
        /// The file this frame comes from
        /// </summary>
        public string file;

        /// <summary>
        /// Construct a new, empty Frame object.
        /// </summary>
        public Frame()
        {

        }
    }
}
