using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;

namespace SpriteSheetPipeline
{
    /// <summary>
    /// This class will be instantiated by the XNA Framework Content Pipeline
    /// to write the specified data type into binary .xnb format.
    ///
    /// This should be part of a Content Pipeline Extension Library project.
    /// </summary>
    [ContentTypeWriter]
    class SpriteSheetWriter : ContentTypeWriter<SpriteSheetMiddleman>
    {
        protected override void Write(ContentWriter output, SpriteSheetMiddleman value)
        {
            //Write files
            output.Write(value.files.Count);
            foreach (File s in value.files)
            {
                output.Write(s.name);
                output.Write(s.normals);
                output.Write(s.width);
                output.Write(s.height);
                output.Write(s.offx);
                output.Write(s.offy);
            }

            //Write animations
            output.Write(value.animations.Count);
            foreach (Animation anim in value.animations)
            {
                output.Write(anim.name);

                //Write frames
                output.Write(anim.frames.Count);
                foreach (Frame frame in anim.frames)
                {
                    output.Write(frame.id);
                    output.Write(frame.length);
                    output.Write(frame.file);
                }
            }
        }

        public override string GetRuntimeType(TargetPlatform targetPlatform)
        {
            return typeof(SpriteSheetMiddleman).AssemblyQualifiedName;
        }

        public override string GetRuntimeReader(TargetPlatform targetPlatform)
        {
            return "Sprites.SpriteSheetReader, SpriteSheets, Version=1.0.0.0, Culture=neutral";
        }
    }
}
